import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter


TEST_ENDPOINT = 'https://wwwcie.ups.com'
PRODUCTION_ENDPOINT = 'https://onlinetools.ups.com'


class UPSApi():
  def __init__(self, merchant_id, client_id, client_secret, environment='production'):
    self.endpoint = TEST_ENDPOINT
    self.environment = environment
    if environment == 'production':
      self.endpoint = PRODUCTION_ENDPOINT

    self.merchant_id = merchant_id
    self.client_id = client_id
    self.client_secret = client_secret
    self.token = None
    self.session = requests.Session()
    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[429, 502, 503, 504],
    )
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))
    self.authorize()

  def get_tracking_details(self, tracking_number):
    url = "{}/api/track/v1/details/{}".format(self.endpoint, tracking_number)
    now = datetime.datetime.utcnow()
    transaction_id = '{}_{}'.format(tracking_number, now.strftime('%Y%m%d%H%M%S'))

    query = {
      "locale": "en_US",
      "returnSignature": "false",
      "returnMilestones": "true"
    }

    headers = {
      "transId": transaction_id,
      "transactionSrc": self.merchant_id,
      "Authorization": "Bearer " + self.token['access_token']
    }

    response = self.session.get(url, headers=headers, params=query)
    return response.json()

  def authorize(self):
    url = "{}/security/v1/oauth/token".format(self.endpoint)

    payload = {
      "grant_type": "client_credentials"
    }

    headers = {
      "Content-Type": "application/x-www-form-urlencoded",
      "x-merchant-id": self.merchant_id
    }

    response = self.session.post(
      url, data=payload, headers=headers, auth=(self.client_id, self.client_secret))
    self.token = response.json()

    if not self.token or 'access_token' not in self.token or not self.token['access_token']:
      raise RuntimeError("Authorization failed")

  @classmethod
  def get_status_by_processing_code(cls, processing_code):
    status =  cls.processing_status_codes().get(processing_code, dict())
    return status.get('status', None)

  @classmethod
  def processing_status_codes(cls):
    return {
      '026': {'status': 'Delivered'},
      '013': {'status': 'Exception'},
      '048': {'status': 'Delay'},
      '005': {'status': 'In Transit'},
      '032': {'status': 'Adverse Weather May Cause Delay'},
      '040': {'status': 'Delivered to UPS Access Point™ Awaiting Customer Pickup'},
      '055': {'status': 'Rescheduled Delivery'},
      '003': {'status': 'Order Processed: Ready for UPS'},
      '012': {'status': 'Clearance in Progress'},
      '065': {'status': 'Pickup Attempted'},
      '057': {'status': 'Package not at UPS Access Point™ yet'},
      '053': {'status': 'Delay: Severe Weather'},
      '006': {'status': 'Out for Delivery Today'},
      '011': {'status': 'Delivered'},
      '054': {'status': 'Delivery Change Requested'},
      '022': {'status': 'First Attempt Made'},
      '024': {'status': 'Final Attempt Made'},
      '016': {'status': 'Held in Warehouse'},
      '018': {'status': 'Delivery Change Requested: Hold for Pickup'},
      '070': {'status': 'Package not at UPS Access Point™ yet'},
      '042': {'status': 'Service Upgraded'},
      '021': {'status': 'Out for Delivery Today'},
      '030': {'status': 'Local Post Office Exception'},
      '035': {'status': 'Returning to Sender'},
      '029': {'status': 'Address Information Required'},
      '028': {'status': 'Delivery Address Changed'},
      '050': {'status': 'Address Information Required'},
      '049': {'status': 'Delay: Attention Needed'},
      '058': {'status': 'Clearance Information Required'},
      '014': {'status': 'Clearance Completed'},
      '051': {'status': 'Delay: Emergency Situation or Severe Weather'},
      '033': {'status': 'Return to Sender Requested'},
      '052': {'status': 'Delay: Severe Weather'},
      '023': {'status': 'Second Attempt Made'},
      '044': {'status': 'On Its Way to UPS'},
      '025': {'status': 'Transferred to Local Post Office for Delivery'},
      '017': {'status': 'Held for Customer Pickup'},
      '019': {'status': 'Delivery Rescheduled'},
      '027': {'status': 'Address Change Requested'},
      '072': {'status': 'Your Driver is Arriving Soon!'},
      '038': {'status': 'Picked Up'},
      '077': {'status': 'Scheduled for Pickup Today'},
      '046': {'status': 'Delay'},
      '047': {'status': 'Delay'},
      '071': {'status': 'Preparing for Delivery Today'},
    }
